@@ -467,7 +467,6 @@ if (allocated(G)) deallocate(G)
 if (allocated(R)) deallocate(R)
 if (allocated(U)) deallocate(U)
 if (allocated(Pi)) deallocate(Pi)
-if (allocated(X0)) deallocate(X0)
 if (allocated(Pi0)) deallocate(Pi0)
 
 m=size(A,1)
@@ -480,13 +479,6 @@ if (info /= 0 ) then
      return
 endif
 
-allocate(X0(m,m),STAT=info)
-if (debug) write (*,*) "qbd_fi_solve:  Allocating X0, stat=",info
-if (info /= 0 ) then
-     errorflag=info
-     return
-endif
-X0=0.0d0
 ds=.false.
 method=3   
 !
@@ -495,32 +487,116 @@ method=3
 !
 select case(algflag)
 case (0) !Natural, Basic, x(0)=0 
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "qbd_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       method = 1
 case (1) !Natural, Basic, x(0)=1
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "qbd_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       method = 1
       do i=1,m
       X0(i,i)=1.0d0
       enddo
 case (2) !Natural, Shift Acceleration
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "qbd_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       method = 1
       ds=.true.
 case (3) !Traditional, Basic, x(0)=0
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "qbd_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       method = 2
 case (4) !Traditional, Basic, x(0)=1
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "qbd_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       method = 2
       do i=1,m
       X0(i,i)=1.0d0
       enddo
 case (5) !Traditional, Shift Acceleration
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "qbd_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       method = 2
       ds=.true.
 case (6) !U-based, Basic, x(0)=0
-
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "qbd_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
 case (7) !U-based, Basic, x(0)=1
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "qbd_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       do i=1,m
       X0(i,i)=1.0d0
       enddo
 case (8) !U-based, Shift Acceleration
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "qbd_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
+      ds=.true.
+case(9) !Natural, Basic, x(0) user
+      method = 1
+case(10) !Traditional, Basic, x(0) user
+      method = 2
+case(11) !U-based, Basic, x(0) user
+case(12) !Natural, Shift Acceleration, x(0) user
+      method = 1
+      ds=.true.
+case(13) !Traditional, Shift Acceleration, x(0) user
+      method = 2
+      ds=.true.
+case(14) !U-based, Shift Acceleration, x(0) user
       ds=.true.
 case default
 if(debug) write(*,*) "qbd_fi_solve: strange algflag=",algflag
@@ -936,7 +1012,6 @@ myeps=epsilon(1.d0) * 10.0d0 ** errmax
 if (allocated(G)) deallocate(G)
 if (allocated(R)) deallocate(R)
 if (allocated(U)) deallocate(U)
-if (allocated(X0)) deallocate(X0)
 if (allocated(Pi)) deallocate(Pi)
 if (allocated(Pi0)) deallocate(Pi0)
 !
@@ -950,13 +1025,6 @@ if (info /= 0 ) then
      return
 endif
 
-allocate(X0(m,m),STAT=info)
-if (debug) write (*,*) "mg1_fi_solve:  Allocating X0, stat=",info
-if (info /= 0 ) then
-     errorflag=info
-     return
-endif
-X0=0.0d0
 ds=.false.
 method=3   
 !
@@ -965,32 +1033,116 @@ method=3
 !
 select case(algflag)
 case (0) !Natural, Basic, x(0)=0 
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "mg1_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       method = 1
 case (1) !Natural, Basic, x(0)=1
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "mg1_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       method = 1
       do i=1,m 
       X0(i,i)=1.0d0
       enddo
 case (2) !Natural, Shift Acceleration
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "mg1_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       method = 1
       ds=.true.
 case (3) !Traditional, Basic, x(0)=0
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "mg1_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       method = 2
 case (4) !Traditional, Basic, x(0)=1
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "mg1_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       method = 2
       do i=1,m
       X0(i,i)=1.0d0
       enddo
 case (5) !Traditional, Shift Acceleration
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "mg1_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       method = 2
       ds=.true.
 case (6) !U-based, Basic, x(0)=0
-
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "mg1_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
 case (7) !U-based, Basic, x(0)=1
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "mg1_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       do i=1,m
       X0(i,i)=1.0d0
       enddo
 case (8) !U-based, Shift Acceleration
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "mg1_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
+      ds=.true.
+case(9) !Natural, Basic, x(0) user
+      method = 1
+case(10) !Traditional, Basic, x(0) user
+      method = 2
+case(11) !U-based, Basic, x(0) user
+case(12) !Natural, Shift Acceleration, x(0) user
+      method = 1
+      ds=.true.
+case(13) !Traditional, Shift Acceleration, x(0) user
+      method = 2
+      ds=.true.
+case(14) !U-based, Shift Acceleration, x(0) user
       ds=.true.
 case default
 if(debug) write(*,*) "mg1_fi_solve: strange algflag=",algflag
@@ -1454,7 +1606,6 @@ if (allocated(G)) deallocate(G)
 if (allocated(R)) deallocate(R)
 if (allocated(Pi)) deallocate(Pi)
 if (allocated(Pi0)) deallocate(Pi0)
-if (allocated(X0)) deallocate(X0)
 if (allocated(vi)) deallocate(vi)
 if (allocated(v)) deallocate(v)
 if (allocated(AA)) deallocate(AA)
@@ -1477,12 +1628,6 @@ if (info /= 0 ) then
      return
 endif
 
-allocate(X0(m,m),STAT=info)
-if (debug) write (*,*) "gm1_fi_solve:  Allocating X0, stat=",info
-if (info /= 0 ) then
-     errorflag=info
-     return
-endif
 
 allocate(AA(m,m,nba),STAT=info)
 if (debug) write (*,*) "gm1_fi_solve:  Allocating AA, stat=",info
@@ -1506,35 +1651,119 @@ method=3
 !
 select case(algflag)
 case (0) !Natural, Basic, x(0)=0 
+      if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "gm1_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       method = 1
 case (1) !Natural, Basic, x(0)=1
+     if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "gm1_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       method = 1
       do i=1,m 
       X0(i,i)=1.0d0
       enddo
 case (2) !Natural, Shift Acceleration
+     if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "gm1_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       method = 1
       ds=.true.
 case (3) !Traditional, Basic, x(0)=0
+     if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "gm1_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       method = 2
 case (4) !Traditional, Basic, x(0)=1
+     if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "gm1_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       method = 2
       do i=1,m
       X0(i,i)=1.0d0
       enddo
 case (5) !Traditional, Shift Acceleration
+     if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "gm1_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       method = 2
       ds=.true.
 case (6) !U-based, Basic, x(0)=0
-
+     if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "gm1_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
 case (7) !U-based, Basic, x(0)=1
+     if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "gm1_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
       do i=1,m
       X0(i,i)=1.0d0
       enddo
 case (8) !U-based, Shift Acceleration
+     if (allocated(X0)) deallocate(X0)
+      allocate(X0(m,m),STAT=info)
+      if (debug) write (*,*) "gm1_fi_solve:  Allocating X0, stat=",info
+      if (info /= 0 ) then
+          errorflag=info
+          return
+      endif
+      X0=0.0d0
+      ds=.true.
+case(9) !Natural, Basic, x(0) user
+      method = 1
+case(10) !Traditional, Basic, x(0) user
+      method = 2
+case(11) !U-based, Basic, x(0) user
+case(12) !Natural, Shift Acceleration, x(0) user
+      method = 1
+      ds=.true.
+case(13) !Traditional, Shift Acceleration, x(0) user
+      method = 2
+      ds=.true.
+case(14) !U-based, Shift Acceleration, x(0) user
       ds=.true.
 case default
-if(debug) write(*,*) "mg1_fi_solve: strange algflag=",algflag
+if(debug) write(*,*) "gm1_fi_solve: strange algflag=",algflag
 end select
 !
 ! setup goals
