@@ -65,10 +65,13 @@ void on_main_menu_algorithms_logarithmicr_shift (GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_logarithmicr_gth (GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_funciter_basic0 (GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_funciter_basic1 (GtkMenuItem *, gpointer );
+void on_main_menu_algorithms_funciter_basic_u (GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_funciter_shift (GtkMenuItem *, gpointer );
+void on_main_menu_algorithms_funciter_shift_u (GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_funciter_natural (GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_funciter_traditional (GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_funciter_Ubased (GtkMenuItem *, gpointer );
+void on_main_menu_algorithms_funciter_readX(GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_invarsubsp_basic (GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_invarsubsp_balzer (GtkMenuItem *, gpointer );
 void on_main_menu_algorithms_invarsubsp_schur (GtkMenuItem *, gpointer );
@@ -166,6 +169,7 @@ int is_edit_b_open;  /* 0=not open  1=open */
 int is_view_matrix_open; /* 0=not open  1=open */
 int is_view_pi_open; /* 0=not open  1=open */
 int is_pi_parameters_open; /* 0=not open  1=open */
+int is_readX_open; /* 0=not open  1=open */
 int is_running;      /* 0 = not running 1 = running */
 int is_completed;      /* 1 if computation did complete 0 if it is not completed yet */
 int is_situation;    /* 0 = not show 1= show situation bar */
@@ -194,7 +198,7 @@ double beta_in;          /* used bye examples window */
 struct algorithms_str algorithms_choice;
 struct goals_str goals_choice;
 struct running_problem_str committed, selected, completed; /* we save data here after successfully allocate the matrices with the data in committed */
-struct edita_str pea;   /* preferences edit_a window */
+struct edita_str pea,peb;   /* preferences edit_a window */
 struct edita_str pview; /* preferences view windows */
 char digits_to_save[64]; /* string corresponding to the number format chosen to write in files */
 
