@@ -465,6 +465,60 @@ md=size(U,2)
    end do
  end do
 end subroutine dump_matU
+
+!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+!
+subroutine alloc_matX0(m,stat)
+use ponte_f_f
+implicit none
+integer m,stat
+if (allocated(X0)) deallocate(X0)
+allocate(X0(m,m),STAT=stat)
+G=0.0d0
+end subroutine alloc_matX0
+
+!
+! to read the content of X0 from C routines
+!
+subroutine read_matX0(n,m,x)
+use ponte_f_f
+implicit none
+real(KIND=8) :: x
+integer n,m
+x=X0(n,m)
+end subroutine read_matX0
+
+!
+! to write to X0 from C routines
+!
+subroutine write_matX0(n,m,x)
+use ponte_f_f
+implicit none
+real(KIND=8) :: x
+integer n,m
+X0(n,m)=x
+end subroutine write_matX0
+
+!
+! test program
+! dumps X0 to Main Window NotePad
+!
+subroutine dump_matX0()
+use ponte_f_f
+implicit none
+integer n,m,nd,md
+! reads dimension nd,md of X0
+nd=size(X0,1)
+md=size(X0,2)
+! writes all the matrix to notePad
+ do m=1,md
+   do n=1,nd
+   write(wout,*)"X0(",n,m,")=",X0(n,m)
+   call print_it_
+   end do
+ end do
+end subroutine dump_matX0
+
 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 !
 subroutine alloc_matPi0(m,stat)
